/*

Read cat-tailed raw measurement files from QCD programs, and 
extract just the quark bilinear data.  Ultimately, it should 
be able to merge data from simulation (light meson) and 
analysis (heavy and heavy/light meson) runs.

*/

#define LINT_ARGS 1      /* Enable Compiler Parameter Checking */
#include <stdio.h>

/*
#define SIMULATION_RUN 
*/
/*
#define SIMPLE_ANALYSIS_RUN 
*/
#define MIXED_ANALYSIS_RUN 

#define NUM_OPERATORS 2  /* I our case, different smearing regimes */

#define DIMENSION   4
#define SPN_SIZE    4
#define COL_SIZE    3

#define MAX_LENGTH 48
#define MAX_MASSES 16

int get_parameters (size, beta,
                    num_updates,
                    smear_radius,
                    num_masses, k_fermion,
                    r_fermion,
                    criteria,
		    heavy_k)
int *size ;
float *beta ;
int *num_updates, *smear_radius, *num_masses ;
float *k_fermion, *r_fermion, *criteria, *heavy_k ;
/*
Read run parameters from standard input
*/
{
  int i ;

  for (i = 0; i < (DIMENSION + 1); i++)
    scanf ("%d", size + i) ;
  scanf ("%d" , num_updates) ;
  scanf ("%d",    smear_radius) ;
  scanf ("%d",    num_masses) ;
  scanf ("%f", beta) ; 
  if (*num_masses > MAX_MASSES)
  {
    printf ("\nERROR:  Too many mass values\n") ;
    return (0) ;
  }
  else
  {
    for (i = 0; i < *num_masses; i++)
      scanf ("%f",  k_fermion + i) ; 
    scanf ("%f",    r_fermion) ;
    scanf ("%f",    criteria) ;
    scanf ("%f",    criteria + 1) ;
#ifdef MIXED_ANALYSIS_RUN
    scanf ("%f",    heavy_k) ;
#endif
    return (1) ;
  }
}

int get_bilins (length, num_masses)
int length, num_masses ;
{

  if (length <= MAX_LENGTH)
  {
    int mass, t, s0, c0, st, o ;

#ifdef SIMULATION_RUN

    {

      /* Read in (ignore) gauge field stuff */

      float plaquette, i_obj, f_obj ;
      int num_iters ;

      scanf ("%f", &plaquette) ;
      scanf ("%d %f %f", &num_iters, &i_obj, &f_obj) ;
    }

#endif



    /* Forward raw bilinear data */

    for (s0 = 0; s0 < SPN_SIZE; s0++)
      for (c0 = 0; c0 < COL_SIZE; c0++)
      {
#ifndef MIXED_ANALYSIS_RUN
        for (mass = 0; mass < num_masses; mass++)
#else

	{

	  /* Read in (ignore) number of iterations used for heavy */

	  int num_iters, result ;

	  result = scanf ("%d", &num_iters) ;
	  if (result == EOF)
	    return 0 ;
	}

        for (mass = 0; mass < num_masses + 1; mass++)
#endif
        {

#ifdef SIMULATION_RUN

          {

            /* Read in (ignore) number of iterations used */

            int num_iters, result ;

            result = scanf ("%d", &num_iters) ;
            if (result == EOF)
              return 0 ;
          }

#endif

          for (o = 0; o < NUM_OPERATORS; o++)
            for (t = 0; t < length; t++)
              for (st = 0; st < SPN_SIZE; st++)
              {
                float re_bilin, im_bilin ;
                int result ;

                result = scanf ("%f %f", &re_bilin, &im_bilin) ;

                if (result == EOF)
                {

#ifdef SIMULATION_RUN

                  printf ("\nERROR: Unexpected end of file\n") ;

#else

                  if (mass | t | s0 | c0 | st)
                    printf ("\nERROR: Unexpected end of file\n") ;

#endif

                  return 0 ;
                }
                if ((st % 2) == 0)
                  printf ("\n") ;  /* Cosmetic */
                printf ("%13e %13e  ", re_bilin, im_bilin) ;
              }
          printf ("\n") ;
        }
      }

    printf ("\n") ;
    return 1 ;
  }
  else
  {
    printf ("\nERROR: Lattice too long\n") ;
    return 0 ;
  }
}

int main (argc, argv, envp)
int argc ;
char *argv [] ;
char *envp [] ;
{
  int size [DIMENSION + 1], num_updates, smear_radius, num_masses ;
  float beta, k_fermion [MAX_MASSES], r_fermion, criteria [2] ;
  float heavy_k ;

  if (get_parameters (size, &beta, &num_updates,
    &smear_radius, &num_masses, k_fermion, &r_fermion, criteria, &heavy_k))
  {
    int length = size [0] ;
    int i ;

    printf ("\n") ;
    printf ("%6d\n", length) ;
#ifndef MIXED_ANALYSIS_RUN
    printf ("%6d\n", num_masses) ;
    printf ("\n") ;
    for (i = 0; i < num_masses; i++)
      printf ("%f %f\n", k_fermion [i], k_fermion [i]) ;
#else
    printf ("%6d\n", num_masses + 1) ;
    printf ("\n") ;
    printf ("%f %f\n", heavy_k, heavy_k) ;
    for (i = 0; i < num_masses; i++)
      printf ("%f %f\n", heavy_k, k_fermion [i]) ;
#endif
    printf ("\n") ;
    while (get_bilins (length, num_masses))
      ; 
  }
}

/*
--
--     End of File.
--
*/
